# TM API

This project uses Quarkus, the Supersonic Subatomic Java Framework.

If you want to learn more about Quarkus, please visit its website: https://quarkus.io/ .

## This project creation

```shell script
quarkus create app --no-code com.ociotec:tm-api:0.1.0
```

## Running the application in dev mode

You can run your application in dev mode that enables live coding using:
```shell script
./mvnw compile quarkus:dev
```

> **_NOTE:_**  Quarkus now ships with a Dev UI, which is available in dev mode only at http://localhost:8080/q/dev/.

## Packaging and running the application

The application can be packaged using:
```shell script
./mvnw package
```
It produces the `quarkus-run.jar` file in the `target/quarkus-app/` directory.
Be aware that it’s not an _über-jar_ as the dependencies are copied into the `target/quarkus-app/lib/` directory.

The application is now runnable using `java -jar target/quarkus-app/quarkus-run.jar`.

If you want to build an _über-jar_, execute the following command:
```shell script
./mvnw package -Dquarkus.package.jar.type=uber-jar
```

The application, packaged as an _über-jar_, is now runnable using `java -jar target/*-runner.jar`.

## Creating a native executable

You can create a native executable using: 
```shell script
./mvnw package -Dnative
```

Or, if you don't have GraalVM installed, you can run the native executable build in a container using: 
```shell script
./mvnw package -Dnative -Dquarkus.native.container-build=true
```

You can then execute your native executable with: `./target/tm-api-0.1.0-runner`

If you want to learn more about building native executables, please consult https://quarkus.io/guides/maven-tooling.

## API

### Query TM samples

GraphQL query TM samples:

```graphql
query {
  samples(system: "SAT10", subsystem: "COOLER", name: "VALUE") {
    id
    system
    subsystem
    name
    timestamp
    value {
      __typename
      ... on ValueInt {int: value}
      ... on ValueLong {long: value}
      ... on ValueString {string: value}
      ... on ValueBoolean {boolean: value}
      ... on ValueFloat {float: value}
      ... on ValueDouble {double: value}
      ... on ValueTimestamp {timestamp: value}
    }
  }
}
```

It will return this data:

```json
{
  "data": {
    "samples": [
      {
        "id": "4100a57d-4d2a-42aa-8cf0-ee704c784291",
        "system": "SAT10",
        "subsystem": "COOLER",
        "name": "VALUE",
        "timestamp": "2024-05-12T21:39:25.005494Z",
        "value": {
          "__typename": "ValueTimestamp",
          "timestamp": "2024-05-12T21:39:24.251102Z"
        }
      },
      {
        "id": "1de43a0c-af6f-47c1-8868-4ea0f05d93be",
        "system": "SAT10",
        "subsystem": "COOLER",
        "name": "VALUE",
        "timestamp": "2024-05-12T21:39:26.002054Z",
        "value": {
          "__typename": "ValueTimestamp",
          "timestamp": "2024-05-12T21:39:24.251102Z"
        }
      },
      {
        "id": "94a02beb-2c95-4b70-a26b-c3a17b8555b3",
        "system": "SAT10",
        "subsystem": "COOLER",
        "name": "VALUE",
        "timestamp": "2024-05-12T21:39:27.002037Z",
        "value": {
          "__typename": "ValueTimestamp",
          "timestamp": "2024-05-12T21:39:24.251102Z"
        }
      }
    ]
  }
}
```

This query can be done also filtering by properties, for example:

```graphql
query {
  samples(subsystem: "CLOCK", properties: [{key: "system", value: {stringValue: "SAT02"}}, {key: "index", value: {intValue: 157}}]) {
    id
    system
    subsystem
    name
    timestamp
    value {
      __typename
      ... on ValueInt {int: value}
      ... on ValueLong {long: value}
      ... on ValueString {string: value}
      ... on ValueBoolean {boolean: value}
      ... on ValueFloat {float: value}
      ... on ValueDouble {double: value}
      ... on ValueTimestamp {timestamp: value}
    }
    properties {
      key
      value {
        __typename
        ... on ValueInt {int: value}
        ... on ValueLong {long: value}
        ... on ValueString {string: value}
        ... on ValueBoolean {boolean: value}
        ... on ValueFloat {float: value}
        ... on ValueDouble {double: value}
        ... on ValueTimestamp {timestamp: value}
      }
    }
  }
}
```

It will return this data:

```json
{
  "data": {
    "samples": [
      {
        "id": "c6399f36-8a5f-4174-b057-255effe586d2",
        "system": "SAT02",
        "subsystem": "CLOCK",
        "name": "ANGLE",
        "timestamp": "2024-05-16T14:52:13.001385Z",
        "value": {
          "__typename": "ValueBoolean",
          "boolean": true
        },
        "properties": [
          {
            "key": "name",
            "value": {
              "__typename": "ValueString",
              "string": "ANGLE"
            }
          },
          {
            "key": "index",
            "value": {
              "__typename": "ValueLong",
              "long": 157
            }
          },
          {
            "key": "system",
            "value": {
              "__typename": "ValueString",
              "string": "SAT02"
            }
          },
          {
            "key": "is_number",
            "value": {
              "__typename": "ValueBoolean",
              "boolean": false
            }
          },
          {
            "key": "is_string",
            "value": {
              "__typename": "ValueBoolean",
              "boolean": false
            }
          },
          {
            "key": "subsystem",
            "value": {
              "__typename": "ValueString",
              "string": "CLOCK"
            }
          }
        ]
      }
    ]
  }
}
```

### Query last live TM sample

GraphQL query live last TM sample:

```graphql
query {
  lastLiveSample {
    id
    system
    subsystem
    name
    timestamp
    value {
      __typename
      ... on ValueInt {int: value}
      ... on ValueLong {long: value}
      ... on ValueString {string: value}
      ... on ValueBoolean {boolean: value}
      ... on ValueFloat {float: value}
      ... on ValueDouble {double: value}
      ... on ValueTimestamp {timestamp: value}
    }
  }
}
```

It will return this data:

```json
{
  "data": {
    "lastLiveSample": {
      "id": "1ab28a10-d73e-4922-9d93-f934ffdc3dae",
      "system": "SAT10",
      "subsystem": "HEATER",
      "name": "GRADIENT",
      "timestamp": "2024-05-12T22:15:18.000901Z",
      "value": {
        "__typename": "ValueString",
        "string": "bhSiZbUMU4"
      }
    }
  }
}
```

### Subscription to new TM samples

GraphQL subscription (including definition for variant values):

```graphql
subscription {
  newSamples(system: "SAT10", subsystem: "COOLER", name: "VALUE") {
    id
    system
    subsystem
    name
    timestamp
    value {
      __typename
      ... on ValueInt {int: value}
      ... on ValueLong {long: value}
      ... on ValueString {string: value}
      ... on ValueBoolean {boolean: value}
      ... on ValueFloat {float: value}
      ... on ValueDouble {double: value}
      ... on ValueTimestamp {timestamp: value}
    }
  }
}
```

It will return this data periodically:

```json
{
  "data": {
    "newSamples": [
      {
        "id": "2768a01d-3a25-46cd-8ebd-61ebbecf1f62",
        "system": "SAT10",
        "subsystem": "COOLER",
        "name": "VALUE",
        "timestamp": "2024-05-12T22:12:24.000910Z",
        "value": {
          "__typename": "ValueDouble",
          "double": 0.5433255372983395
        }
      }
    ]
  }
}
```
